@@ -23,7 +23,6 @@ import re
 import tarfile
 import tempfile
 import time
-import warnings
 from collections import Counter
 from datetime import datetime
 from functools import partial
@@ -35,6 +34,7 @@ from airflow.exceptions import AirflowException
 from airflow.providers.amazon.aws.hooks.base_aws import AwsBaseHook
 from airflow.providers.amazon.aws.hooks.logs import AwsLogsHook
 from airflow.providers.amazon.aws.hooks.s3 import S3Hook
+from airflow.providers.amazon.aws.utils.tags import format_tags
 from airflow.utils import timezone
 
 
@@ -137,12 +137,13 @@ def secondary_training_status_message(
 class SageMakerHook(AwsBaseHook):
     """
     Interact with Amazon SageMaker.
+    Provide thick wrapper around :external+boto3:py:class:`boto3.client("sagemaker") <SageMaker.Client>`.
 
     Additional arguments (such as ``aws_conn_id``) may be specified and
     are passed down to the underlying AwsBaseHook.
 
     .. seealso::
-        :class:`~airflow.providers.amazon.aws.hooks.base_aws.AwsBaseHook`
+        - :class:`airflow.providers.amazon.aws.hooks.base_aws.AwsBaseHook`
     """
 
     non_terminal_states = {"InProgress", "Stopping"}
@@ -370,6 +371,9 @@ class SageMakerHook(AwsBaseHook):
         Starts a transform job. A transform job uses a trained model to get inferences
         on a dataset and saves these results to an Amazon S3 location that you specify.
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.create_transform_job`
+
         :param config: the config for transform job
         :param wait_for_completion: if the program should keep running until job finishes
         :param check_interval: the time interval in seconds which the operator
@@ -406,6 +410,9 @@ class SageMakerHook(AwsBaseHook):
         experience on SageMaker to run your data processing workloads, such as feature
         engineering, data validation, model evaluation, and model interpretation.
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.create_processing_job`
+
         :param config: the config for processing job
         :param wait_for_completion: if the program should keep running until job finishes
         :param check_interval: the time interval in seconds which the operator
@@ -433,6 +440,9 @@ class SageMakerHook(AwsBaseHook):
         image that contains inference code, artifacts (from prior training), and a custom
         environment map that the inference code uses when you deploy the model for predictions.
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.create_model`
+
         :param config: the config for model
         :return: A response to model creation
         """
@@ -446,8 +456,9 @@ class SageMakerHook(AwsBaseHook):
         the resources that you want Amazon SageMaker to provision.
 
         .. seealso::
-             :class:`~airflow.providers.amazon.aws.hooks.sagemaker.SageMakerHook.create_model`
-             :class:`~airflow.providers.amazon.aws.hooks.sagemaker.SageMakerHook.create_endpoint`
+            - :external+boto3:py:meth:`SageMaker.Client.create_endpoint_config`
+            - :class:`airflow.providers.amazon.aws.hooks.sagemaker.SageMakerHook.create_model`
+            - :class:`airflow.providers.amazon.aws.hooks.sagemaker.SageMakerHook.create_endpoint`
 
         :param config: the config for endpoint-config
         :return: A response to endpoint config creation
@@ -468,9 +479,10 @@ class SageMakerHook(AwsBaseHook):
         the compute resources up and down as needed to handle your request traffic.
 
         Requires an Endpoint Config.
-         .. seealso::
-             :class:`~airflow.providers.amazon.aws.hooks.sagemaker.SageMakerHook.create_endpoint_config`
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.create_endpoint`
+            - :class:`airflow.providers.amazon.aws.hooks.sagemaker.SageMakerHook.create_endpoint`
 
         :param config: the config for endpoint
         :param wait_for_completion: if the program should keep running until job finishes
@@ -505,6 +517,9 @@ class SageMakerHook(AwsBaseHook):
         newly created endpoint, and then deletes resources provisioned for the
         endpoint using the previous EndpointConfig (there is no availability loss).
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.update_endpoint`
+
         :param config: the config for endpoint
         :param wait_for_completion: if the program should keep running until job finishes
         :param check_interval: the time interval in seconds which the operator
@@ -530,6 +545,9 @@ class SageMakerHook(AwsBaseHook):
         """
         Return the training job info associated with the name
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.describe_training_job`
+
         :param name: the name of the training job
         :return: A dict contains all the training job info
         """
@@ -600,6 +618,9 @@ class SageMakerHook(AwsBaseHook):
         """
         Return the tuning job info associated with the name
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.describe_hyper_parameter_tuning_job`
+
         :param name: the name of the tuning job
         :return: A dict contains all the tuning job info
         """
@@ -618,6 +639,9 @@ class SageMakerHook(AwsBaseHook):
         """
         Return the transform job info associated with the name
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.describe_transform_job`
+
         :param name: the name of the transform job
         :return: A dict contains all the transform job info
         """
@@ -627,6 +651,9 @@ class SageMakerHook(AwsBaseHook):
         """
         Return the processing job info associated with the name
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.describe_processing_job`
+
         :param name: the name of the processing job
         :return: A dict contains all the processing job info
         """
@@ -636,6 +663,9 @@ class SageMakerHook(AwsBaseHook):
         """
         Return the endpoint config info associated with the name
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.describe_endpoint_config`
+
         :param name: the name of the endpoint config
         :return: A dict contains all the endpoint config info
         """
@@ -643,6 +673,11 @@ class SageMakerHook(AwsBaseHook):
 
     def describe_endpoint(self, name: str) -> dict:
         """
+        Returns the description of an endpoint.
+
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.describe_endpoint`
+
         :param name: the name of the endpoint
         :return: A dict contains all the endpoint info
         """
@@ -805,7 +840,7 @@ class SageMakerHook(AwsBaseHook):
             list_training_jobs(name_contains="myjob", StatusEquals="Failed")
 
         .. seealso::
-            https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/sagemaker.html#SageMaker.Client.list_training_jobs
+            - :external+boto3:py:meth:`SageMaker.Client.list_training_jobs`
 
         :param name_contains: (optional) partial name to match
         :param max_results: (optional) maximum number of results to return. None returns infinite results
@@ -833,7 +868,7 @@ class SageMakerHook(AwsBaseHook):
             list_transform_jobs(name_contains="myjob", StatusEquals="Failed")
 
         .. seealso::
-            https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/sagemaker.html#SageMaker.Client.list_transform_jobs
+            - :external+boto3:py:meth:`SageMaker.Client.list_transform_jobs`
 
         :param name_contains: (optional) partial name to match
         :param max_results: (optional) maximum number of results to return. None returns infinite results
@@ -857,7 +892,7 @@ class SageMakerHook(AwsBaseHook):
             list_processing_jobs(NameContains="myjob", StatusEquals="Failed")
 
         .. seealso::
-            https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/sagemaker.html#SageMaker.Client.list_processing_jobs
+            - :external+boto3:py:meth:`SageMaker.Client.list_processing_jobs`
 
         :param kwargs: (optional) kwargs to boto3's list_training_jobs method
         :return: results of the list_processing_jobs request
@@ -941,21 +976,6 @@ class SageMakerHook(AwsBaseHook):
             else:
                 next_token = response["NextToken"]
 
-    def find_processing_job_by_name(self, processing_job_name: str) -> bool:
-        """
-        Query processing job by name
-
-        This method is deprecated.
-        Please use `airflow.providers.amazon.aws.hooks.sagemaker.count_processing_jobs_by_name`.
-        """
-        warnings.warn(
-            "This method is deprecated. "
-            "Please use `airflow.providers.amazon.aws.hooks.sagemaker.count_processing_jobs_by_name`.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        return bool(self.count_processing_jobs_by_name(processing_job_name))
-
     @staticmethod
     def _name_matches_pattern(
         processing_job_name: str,
@@ -1002,7 +1022,11 @@ class SageMakerHook(AwsBaseHook):
             raise
 
     def delete_model(self, model_name: str):
-        """Delete SageMaker model
+        """
+        Delete SageMaker model.
+
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.delete_model`
 
         :param model_name: name of the model
         """
@@ -1013,7 +1037,12 @@ class SageMakerHook(AwsBaseHook):
             raise
 
     def describe_pipeline_exec(self, pipeline_exec_arn: str, verbose: bool = False):
-        """Get info about a SageMaker pipeline execution
+        """
+        Get info about a SageMaker pipeline execution.
+
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.describe_pipeline_execution`
+            - :external+boto3:py:meth:`SageMaker.Client.list_pipeline_execution_steps`
 
         :param pipeline_exec_arn: arn of the pipeline execution
         :param verbose: Whether to log details about the steps status in the pipeline execution
@@ -1039,7 +1068,10 @@ class SageMakerHook(AwsBaseHook):
         verbose: bool = True,
     ) -> str:
         """
-        Start a new execution for a SageMaker pipeline
+        Start a new execution for a SageMaker pipeline.
+
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.start_pipeline_execution`
 
         :param pipeline_name: Name of the pipeline to start (this is _not_ the ARN).
         :param display_name: The name this pipeline execution will have in the UI. Doesn't need to be unique.
@@ -1053,9 +1085,7 @@ class SageMakerHook(AwsBaseHook):
 
         :return: the ARN of the pipeline execution launched.
         """
-        if pipeline_params is None:
-            pipeline_params = {}
-        formatted_params = [{"Name": kvp[0], "Value": kvp[1]} for kvp in pipeline_params.items()]
+        formatted_params = format_tags(pipeline_params, key_label="Name")
 
         try:
             res = self.conn.start_pipeline_execution(
@@ -1088,6 +1118,9 @@ class SageMakerHook(AwsBaseHook):
     ) -> str:
         """Stop SageMaker pipeline execution
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.stop_pipeline_execution`
+
         :param pipeline_exec_arn: Amazon Resource Name (ARN) of the pipeline execution.
             It's the ARN of the pipeline itself followed by "/execution/" and an id.
         :param wait_for_completion: Whether to wait for the pipeline to reach a final state.
@@ -1104,19 +1137,35 @@ class SageMakerHook(AwsBaseHook):
         :return: Status of the pipeline execution after the operation.
             One of 'Executing'|'Stopping'|'Stopped'|'Failed'|'Succeeded'.
         """
-        try:
-            self.conn.stop_pipeline_execution(PipelineExecutionArn=pipeline_exec_arn)
-        except ClientError as ce:
-            # we have to rely on the message to catch the right error here, because its type
-            # (ValidationException) is shared with other kinds of error (for instance, badly formatted ARN)
-            if (
-                not fail_if_not_running
-                and "Only pipelines with 'Executing' status can be stopped" in ce.response["Error"]["Message"]
-            ):
-                self.log.warning("Cannot stop pipeline execution, as it was not running: %s", ce)
-            else:
-                self.log.error(ce)
-                raise
+        retries = 2  # i.e. 3 calls max, 1 initial + 2 retries
+        while True:
+            try:
+                self.conn.stop_pipeline_execution(PipelineExecutionArn=pipeline_exec_arn)
+                break
+            except ClientError as ce:
+                # this can happen if the pipeline was transitioning between steps at that moment
+                if ce.response["Error"]["Code"] == "ConflictException" and retries > 0:
+                    retries = retries - 1
+                    self.log.warning(
+                        "Got a conflict exception when trying to stop the pipeline, "
+                        "retrying %s more times. Error was: %s",
+                        retries,
+                        ce,
+                    )
+                    time.sleep(0.3)  # error is due to a race condition, so it should be very transient
+                    continue
+                # we have to rely on the message to catch the right error here, because its type
+                # (ValidationException) is shared with other kinds of errors (e.g. badly formatted ARN)
+                if (
+                    not fail_if_not_running
+                    and "Only pipelines with 'Executing' status can be stopped"
+                    in ce.response["Error"]["Message"]
+                ):
+                    self.log.warning("Cannot stop pipeline execution, as it was not running: %s", ce)
+                else:
+                    self.log.error(ce)
+                    raise
+                break
 
         res = self.describe_pipeline_exec(pipeline_exec_arn)
 
@@ -1135,6 +1184,9 @@ class SageMakerHook(AwsBaseHook):
         """
         Creates a Model Package Group if it does not already exist
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.create_model_package_group`
+
         :param package_group_name: Name of the model package group to create if not already present.
         :param package_group_desc: Description of the model package group, if it was to be created (optional).
 
@@ -1187,6 +1239,9 @@ class SageMakerHook(AwsBaseHook):
         Creates an auto ML job, learning to predict the given column from the data provided through S3.
         The learning output is written to the specified S3 location.
 
+        .. seealso::
+            - :external+boto3:py:meth:`SageMaker.Client.create_auto_ml_job`
+
         :param job_name: Name of the job to create, needs to be unique within the account.
         :param s3_input: The S3 location (folder or file) where to fetch the data.
             By default, it expects csv with headers.
