@@ -17,8 +17,7 @@
 # under the License.
 from __future__ import annotations
 
-import warnings
-from typing import TYPE_CHECKING, Any, Optional, Sequence
+from typing import TYPE_CHECKING, Any, Sequence
 
 from airflow.compat.functools import cached_property
 from airflow.models import BaseOperator
@@ -45,14 +44,15 @@ class AthenaOperator(BaseOperator):
     :param query_execution_context: Context in which query need to be run
     :param result_configuration: Dict with path to store results in and config related to encryption
     :param sleep_time: Time (in seconds) to wait between two consecutive calls to check query status on Athena
-    :param max_tries: Deprecated - use max_polling_attempts instead.
     :param max_polling_attempts: Number of times to poll for query state before function exits
         To limit task execution time, use execution_timeout.
+    :param log_query: Whether to log athena query and other execution params when it's executed.
+        Defaults to *True*.
     """
 
-    ui_color = '#44b5e2'
-    template_fields: Sequence[str] = ('query', 'database', 'output_location')
-    template_ext: Sequence[str] = ('.sql',)
+    ui_color = "#44b5e2"
+    template_fields: Sequence[str] = ("query", "database", "output_location")
+    template_ext: Sequence[str] = (".sql",)
     template_fields_renderers = {"query": "sql"}
 
     def __init__(
@@ -67,8 +67,8 @@ class AthenaOperator(BaseOperator):
         query_execution_context: dict[str, str] | None = None,
         result_configuration: dict[str, Any] | None = None,
         sleep_time: int = 30,
-        max_tries: int | None = None,
         max_polling_attempts: int | None = None,
+        log_query: bool = True,
         **kwargs: Any,
     ) -> None:
         super().__init__(**kwargs)
@@ -82,29 +82,18 @@ class AthenaOperator(BaseOperator):
         self.result_configuration = result_configuration or {}
         self.sleep_time = sleep_time
         self.max_polling_attempts = max_polling_attempts
-        self.query_execution_id = None  # type: Optional[str]
-
-        if max_tries:
-            warnings.warn(
-                f"Parameter `{self.__class__.__name__}.max_tries` is deprecated and will be removed "
-                "in a future release.  Please use method `max_polling_attempts` instead.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-            if max_polling_attempts and max_polling_attempts != max_tries:
-                raise Exception("max_polling_attempts must be the same value as max_tries")
-            else:
-                self.max_polling_attempts = max_tries
+        self.query_execution_id: str | None = None
+        self.log_query: bool = log_query
 
     @cached_property
     def hook(self) -> AthenaHook:
         """Create and return an AthenaHook."""
-        return AthenaHook(self.aws_conn_id, sleep_time=self.sleep_time)
+        return AthenaHook(self.aws_conn_id, sleep_time=self.sleep_time, log_query=self.log_query)
 
     def execute(self, context: Context) -> str | None:
         """Run Presto Query on Athena"""
-        self.query_execution_context['Database'] = self.database
-        self.result_configuration['OutputLocation'] = self.output_location
+        self.query_execution_context["Database"] = self.database
+        self.result_configuration["OutputLocation"] = self.output_location
         self.query_execution_id = self.hook.run_query(
             self.query,
             self.query_execution_context,
@@ -112,18 +101,21 @@ class AthenaOperator(BaseOperator):
             self.client_request_token,
             self.workgroup,
         )
-        query_status = self.hook.poll_query_status(self.query_execution_id, self.max_polling_attempts)
+        query_status = self.hook.poll_query_status(
+            self.query_execution_id,
+            max_polling_attempts=self.max_polling_attempts,
+        )
 
         if query_status in AthenaHook.FAILURE_STATES:
             error_message = self.hook.get_state_change_reason(self.query_execution_id)
             raise Exception(
-                f'Final state of Athena job is {query_status}, query_execution_id is '
-                f'{self.query_execution_id}. Error: {error_message}'
+                f"Final state of Athena job is {query_status}, query_execution_id is "
+                f"{self.query_execution_id}. Error: {error_message}"
             )
         elif not query_status or query_status in AthenaHook.INTERMEDIATE_STATES:
             raise Exception(
-                f'Final state of Athena job is {query_status}. Max tries of poll status exceeded, '
-                f'query_execution_id is {self.query_execution_id}.'
+                f"Final state of Athena job is {query_status}. Max tries of poll status exceeded, "
+                f"query_execution_id is {self.query_execution_id}."
             )
 
         return self.query_execution_id
@@ -131,19 +123,20 @@ class AthenaOperator(BaseOperator):
     def on_kill(self) -> None:
         """Cancel the submitted athena query"""
         if self.query_execution_id:
-            self.log.info('Received a kill signal.')
-            self.log.info('Stopping Query with executionId - %s', self.query_execution_id)
+            self.log.info("Received a kill signal.")
             response = self.hook.stop_query(self.query_execution_id)
             http_status_code = None
             try:
-                http_status_code = response['ResponseMetadata']['HTTPStatusCode']
-            except Exception as ex:
-                self.log.error('Exception while cancelling query: %s', ex)
+                http_status_code = response["ResponseMetadata"]["HTTPStatusCode"]
+            except Exception:
+                self.log.exception(
+                    "Exception while cancelling query. Query execution id: %s", self.query_execution_id
+                )
             finally:
                 if http_status_code is None or http_status_code != 200:
-                    self.log.error('Unable to request query cancel on athena. Exiting')
+                    self.log.error("Unable to request query cancel on athena. Exiting")
                 else:
                     self.log.info(
-                        'Polling Athena for query with id %s to reach final state', self.query_execution_id
+                        "Polling Athena for query with id %s to reach final state", self.query_execution_id
                     )
                     self.hook.poll_query_status(self.query_execution_id)
