@@ -18,15 +18,14 @@
 from __future__ import annotations
 
 import json
-import time
 from typing import TYPE_CHECKING, Sequence
 
 from mypy_boto3_rds.type_defs import TagTypeDef
 
-from airflow.exceptions import AirflowException
 from airflow.models import BaseOperator
 from airflow.providers.amazon.aws.hooks.rds import RdsHook
 from airflow.providers.amazon.aws.utils.rds import RdsDbType
+from airflow.providers.amazon.aws.utils.tags import format_tags
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
@@ -45,66 +44,6 @@ class RdsBaseOperator(BaseOperator):
 
         self._await_interval = 60  # seconds
 
-    def _describe_item(self, item_type: str, item_name: str) -> list:
-        if item_type == "instance_snapshot":
-            db_snaps = self.hook.conn.describe_db_snapshots(DBSnapshotIdentifier=item_name)
-            return db_snaps["DBSnapshots"]
-        elif item_type == "cluster_snapshot":
-            cl_snaps = self.hook.conn.describe_db_cluster_snapshots(DBClusterSnapshotIdentifier=item_name)
-            return cl_snaps["DBClusterSnapshots"]
-        elif item_type == "export_task":
-            exports = self.hook.conn.describe_export_tasks(ExportTaskIdentifier=item_name)
-            return exports["ExportTasks"]
-        elif item_type == "event_subscription":
-            subscriptions = self.hook.conn.describe_event_subscriptions(SubscriptionName=item_name)
-            return subscriptions["EventSubscriptionsList"]
-        elif item_type == "db_instance":
-            instances = self.hook.conn.describe_db_instances(DBInstanceIdentifier=item_name)
-            return instances["DBInstances"]
-        elif item_type == "db_cluster":
-            clusters = self.hook.conn.describe_db_clusters(DBClusterIdentifier=item_name)
-            return clusters["DBClusters"]
-        else:
-            raise AirflowException(f"Method for {item_type} is not implemented")
-
-    def _await_status(
-        self,
-        item_type: str,
-        item_name: str,
-        wait_statuses: list[str] | None = None,
-        ok_statuses: list[str] | None = None,
-        error_statuses: list[str] | None = None,
-    ) -> None:
-        """
-        Continuously gets item description from `_describe_item()` and waits while:
-        - status is in `wait_statuses`
-        - status not in `ok_statuses` and `error_statuses`
-        """
-        while True:
-            items = self._describe_item(item_type, item_name)
-
-            if len(items) == 0:
-                raise AirflowException(f"There is no {item_type} with identifier {item_name}")
-            if len(items) > 1:
-                raise AirflowException(f"There are {len(items)} {item_type} with identifier {item_name}")
-
-            if item_type == "db_instance":
-                status_field = "DBInstanceStatus"
-            else:
-                status_field = "Status"
-
-            if wait_statuses and items[0][status_field].lower() in wait_statuses:
-                time.sleep(self._await_interval)
-                continue
-            elif ok_statuses and items[0][status_field].lower() in ok_statuses:
-                break
-            elif error_statuses and items[0][status_field].lower() in error_statuses:
-                raise AirflowException(f"Item has error status ({error_statuses}): {items[0]}")
-            else:
-                raise AirflowException(f"Item has uncertain status: {items[0]}")
-
-        return None
-
     def execute(self, context: Context) -> str:
         """Different implementations for snapshots, tasks and events"""
         raise NotImplementedError
@@ -126,7 +65,7 @@ class RdsCreateDbSnapshotOperator(RdsBaseOperator):
     :param db_type: Type of the DB - either "instance" or "cluster"
     :param db_identifier: The identifier of the instance or cluster that you want to create the snapshot of
     :param db_snapshot_identifier: The identifier for the DB snapshot
-    :param tags: A list of tags in format `[{"Key": "something", "Value": "something"},]
+    :param tags: A dictionary of tags or a list of tags in format `[{"Key": "...", "Value": "..."},]`
         `USER Tagging <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html>`__
     :param wait_for_completion:  If True, waits for creation of the DB snapshot to complete. (default: True)
     """
@@ -139,16 +78,16 @@ class RdsCreateDbSnapshotOperator(RdsBaseOperator):
         db_type: str,
         db_identifier: str,
         db_snapshot_identifier: str,
-        tags: Sequence[TagTypeDef] | None = None,
+        tags: Sequence[TagTypeDef] | dict | None = None,
         wait_for_completion: bool = True,
-        aws_conn_id: str = "aws_conn_id",
+        aws_conn_id: str = "aws_default",
         **kwargs,
     ):
         super().__init__(aws_conn_id=aws_conn_id, **kwargs)
         self.db_type = RdsDbType(db_type)
         self.db_identifier = db_identifier
         self.db_snapshot_identifier = db_snapshot_identifier
-        self.tags = tags or []
+        self.tags = tags
         self.wait_for_completion = wait_for_completion
 
     def execute(self, context: Context) -> str:
@@ -159,31 +98,27 @@ class RdsCreateDbSnapshotOperator(RdsBaseOperator):
             self.db_snapshot_identifier,
         )
 
+        formatted_tags = format_tags(self.tags)
         if self.db_type.value == "instance":
             create_instance_snap = self.hook.conn.create_db_snapshot(
                 DBInstanceIdentifier=self.db_identifier,
                 DBSnapshotIdentifier=self.db_snapshot_identifier,
-                Tags=self.tags,
+                Tags=formatted_tags,
             )
             create_response = json.dumps(create_instance_snap, default=str)
-            item_type = "instance_snapshot"
-
+            if self.wait_for_completion:
+                self.hook.wait_for_db_snapshot_state(self.db_snapshot_identifier, target_state="available")
         else:
             create_cluster_snap = self.hook.conn.create_db_cluster_snapshot(
                 DBClusterIdentifier=self.db_identifier,
                 DBClusterSnapshotIdentifier=self.db_snapshot_identifier,
-                Tags=self.tags,
+                Tags=formatted_tags,
             )
             create_response = json.dumps(create_cluster_snap, default=str)
-            item_type = "cluster_snapshot"
-
-        if self.wait_for_completion:
-            self._await_status(
-                item_type,
-                self.db_snapshot_identifier,
-                wait_statuses=["creating"],
-                ok_statuses=["available"],
-            )
+            if self.wait_for_completion:
+                self.hook.wait_for_db_cluster_snapshot_state(
+                    self.db_snapshot_identifier, target_state="available"
+                )
         return create_response
 
 
@@ -199,7 +134,7 @@ class RdsCopyDbSnapshotOperator(RdsBaseOperator):
     :param source_db_snapshot_identifier: The identifier of the source snapshot
     :param target_db_snapshot_identifier: The identifier of the target snapshot
     :param kms_key_id: The AWS KMS key identifier for an encrypted DB snapshot
-    :param tags: A list of tags in format `[{"Key": "something", "Value": "something"},]
+    :param tags: A dictionary of tags or a list of tags in format `[{"Key": "...", "Value": "..."},]`
         `USER Tagging <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html>`__
     :param copy_tags: Whether to copy all tags from the source snapshot to the target snapshot (default False)
     :param pre_signed_url: The URL that contains a Signature Version 4 signed request
@@ -226,7 +161,7 @@ class RdsCopyDbSnapshotOperator(RdsBaseOperator):
         source_db_snapshot_identifier: str,
         target_db_snapshot_identifier: str,
         kms_key_id: str = "",
-        tags: Sequence[TagTypeDef] | None = None,
+        tags: Sequence[TagTypeDef] | dict | None = None,
         copy_tags: bool = False,
         pre_signed_url: str = "",
         option_group_name: str = "",
@@ -242,7 +177,7 @@ class RdsCopyDbSnapshotOperator(RdsBaseOperator):
         self.source_db_snapshot_identifier = source_db_snapshot_identifier
         self.target_db_snapshot_identifier = target_db_snapshot_identifier
         self.kms_key_id = kms_key_id
-        self.tags = tags or []
+        self.tags = tags
         self.copy_tags = copy_tags
         self.pre_signed_url = pre_signed_url
         self.option_group_name = option_group_name
@@ -257,12 +192,13 @@ class RdsCopyDbSnapshotOperator(RdsBaseOperator):
             self.target_db_snapshot_identifier,
         )
 
+        formatted_tags = format_tags(self.tags)
         if self.db_type.value == "instance":
             copy_instance_snap = self.hook.conn.copy_db_snapshot(
                 SourceDBSnapshotIdentifier=self.source_db_snapshot_identifier,
                 TargetDBSnapshotIdentifier=self.target_db_snapshot_identifier,
                 KmsKeyId=self.kms_key_id,
-                Tags=self.tags,
+                Tags=formatted_tags,
                 CopyTags=self.copy_tags,
                 PreSignedUrl=self.pre_signed_url,
                 OptionGroupName=self.option_group_name,
@@ -270,28 +206,25 @@ class RdsCopyDbSnapshotOperator(RdsBaseOperator):
                 SourceRegion=self.source_region,
             )
             copy_response = json.dumps(copy_instance_snap, default=str)
-            item_type = "instance_snapshot"
-
+            if self.wait_for_completion:
+                self.hook.wait_for_db_snapshot_state(
+                    self.target_db_snapshot_identifier, target_state="available"
+                )
         else:
             copy_cluster_snap = self.hook.conn.copy_db_cluster_snapshot(
                 SourceDBClusterSnapshotIdentifier=self.source_db_snapshot_identifier,
                 TargetDBClusterSnapshotIdentifier=self.target_db_snapshot_identifier,
                 KmsKeyId=self.kms_key_id,
-                Tags=self.tags,
+                Tags=formatted_tags,
                 CopyTags=self.copy_tags,
                 PreSignedUrl=self.pre_signed_url,
                 SourceRegion=self.source_region,
             )
             copy_response = json.dumps(copy_cluster_snap, default=str)
-            item_type = "cluster_snapshot"
-
-        if self.wait_for_completion:
-            self._await_status(
-                item_type,
-                self.target_db_snapshot_identifier,
-                wait_statuses=["creating", "copying"],
-                ok_statuses=["available"],
-            )
+            if self.wait_for_completion:
+                self.hook.wait_for_db_cluster_snapshot_state(
+                    self.target_db_snapshot_identifier, target_state="available"
+                )
         return copy_response
 
 
@@ -314,6 +247,7 @@ class RdsDeleteDbSnapshotOperator(RdsBaseOperator):
         *,
         db_type: str,
         db_snapshot_identifier: str,
+        wait_for_completion: bool = True,
         aws_conn_id: str = "aws_default",
         **kwargs,
     ):
@@ -321,6 +255,7 @@ class RdsDeleteDbSnapshotOperator(RdsBaseOperator):
 
         self.db_type = RdsDbType(db_type)
         self.db_snapshot_identifier = db_snapshot_identifier
+        self.wait_for_completion = wait_for_completion
 
     def execute(self, context: Context) -> str:
         self.log.info("Starting to delete snapshot '%s'", self.db_snapshot_identifier)
@@ -330,11 +265,17 @@ class RdsDeleteDbSnapshotOperator(RdsBaseOperator):
                 DBSnapshotIdentifier=self.db_snapshot_identifier,
             )
             delete_response = json.dumps(delete_instance_snap, default=str)
+            if self.wait_for_completion:
+                self.hook.wait_for_db_snapshot_state(self.db_snapshot_identifier, target_state="deleted")
         else:
             delete_cluster_snap = self.hook.conn.delete_db_cluster_snapshot(
                 DBClusterSnapshotIdentifier=self.db_snapshot_identifier,
             )
             delete_response = json.dumps(delete_cluster_snap, default=str)
+            if self.wait_for_completion:
+                self.hook.wait_for_db_cluster_snapshot_state(
+                    self.db_snapshot_identifier, target_state="deleted"
+                )
 
         return delete_response
 
@@ -406,14 +347,7 @@ class RdsStartExportTaskOperator(RdsBaseOperator):
         )
 
         if self.wait_for_completion:
-            self._await_status(
-                "export_task",
-                self.export_task_identifier,
-                wait_statuses=["starting", "in_progress"],
-                ok_statuses=["complete"],
-                error_statuses=["canceling", "canceled"],
-            )
-
+            self.hook.wait_for_export_task_state(self.export_task_identifier, target_state="complete")
         return json.dumps(start_export, default=str)
 
 
@@ -436,6 +370,7 @@ class RdsCancelExportTaskOperator(RdsBaseOperator):
         *,
         export_task_identifier: str,
         wait_for_completion: bool = True,
+        check_interval: int = 30,
         aws_conn_id: str = "aws_default",
         **kwargs,
     ):
@@ -443,6 +378,7 @@ class RdsCancelExportTaskOperator(RdsBaseOperator):
 
         self.export_task_identifier = export_task_identifier
         self.wait_for_completion = wait_for_completion
+        self.check_interval = check_interval
 
     def execute(self, context: Context) -> str:
         self.log.info("Canceling export task %s", self.export_task_identifier)
@@ -452,13 +388,9 @@ class RdsCancelExportTaskOperator(RdsBaseOperator):
         )
 
         if self.wait_for_completion:
-            self._await_status(
-                "export_task",
-                self.export_task_identifier,
-                wait_statuses=["canceling"],
-                ok_statuses=["canceled"],
+            self.hook.wait_for_export_task_state(
+                self.export_task_identifier, target_state="canceled", check_interval=self.check_interval
             )
-
         return json.dumps(cancel_export, default=str)
 
 
@@ -478,7 +410,7 @@ class RdsCreateEventSubscriptionOperator(RdsBaseOperator):
         `USER Events <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Events.Messages.html>`__
     :param source_ids: The list of identifiers of the event sources for which events are returned
     :param enabled: A value that indicates whether to activate the subscription (default True)l
-    :param tags: A list of tags in format `[{"Key": "something", "Value": "something"},]
+    :param tags: A dictionary of tags or a list of tags in format `[{"Key": "...", "Value": "..."},]`
         `USER Tagging <https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/USER_Tagging.html>`__
     :param wait_for_completion:  If True, waits for creation of the subscription to complete. (default: True)
     """
@@ -501,7 +433,7 @@ class RdsCreateEventSubscriptionOperator(RdsBaseOperator):
         event_categories: Sequence[str] | None = None,
         source_ids: Sequence[str] | None = None,
         enabled: bool = True,
-        tags: Sequence[TagTypeDef] | None = None,
+        tags: Sequence[TagTypeDef] | dict | None = None,
         wait_for_completion: bool = True,
         aws_conn_id: str = "aws_default",
         **kwargs,
@@ -514,12 +446,13 @@ class RdsCreateEventSubscriptionOperator(RdsBaseOperator):
         self.event_categories = event_categories or []
         self.source_ids = source_ids or []
         self.enabled = enabled
-        self.tags = tags or []
+        self.tags = tags
         self.wait_for_completion = wait_for_completion
 
     def execute(self, context: Context) -> str:
         self.log.info("Creating event subscription '%s' to '%s'", self.subscription_name, self.sns_topic_arn)
 
+        formatted_tags = format_tags(self.tags)
         create_subscription = self.hook.conn.create_event_subscription(
             SubscriptionName=self.subscription_name,
             SnsTopicArn=self.sns_topic_arn,
@@ -527,17 +460,11 @@ class RdsCreateEventSubscriptionOperator(RdsBaseOperator):
             EventCategories=self.event_categories,
             SourceIds=self.source_ids,
             Enabled=self.enabled,
-            Tags=self.tags,
+            Tags=formatted_tags,
         )
 
         if self.wait_for_completion:
-            self._await_status(
-                "event_subscription",
-                self.subscription_name,
-                wait_statuses=["creating"],
-                ok_statuses=["active"],
-            )
-
+            self.hook.wait_for_event_subscription_state(self.subscription_name, target_state="active")
         return json.dumps(create_subscription, default=str)
 
 
@@ -628,10 +555,7 @@ class RdsCreateDbInstanceOperator(RdsBaseOperator):
         )
 
         if self.wait_for_completion:
-            self.hook.conn.get_waiter("db_instance_available").wait(
-                DBInstanceIdentifier=self.db_instance_identifier
-            )
-
+            self.hook.wait_for_db_instance_state(self.db_instance_identifier, target_state="available")
         return json.dumps(create_db_instance, default=str)
 
 
@@ -675,10 +599,7 @@ class RdsDeleteDbInstanceOperator(RdsBaseOperator):
         )
 
         if self.wait_for_completion:
-            self.hook.conn.get_waiter("db_instance_deleted").wait(
-                DBInstanceIdentifier=self.db_instance_identifier
-            )
-
+            self.hook.wait_for_db_instance_state(self.db_instance_identifier, target_state="deleted")
         return json.dumps(delete_db_instance, default=str)
 
 
@@ -730,9 +651,9 @@ class RdsStartDbOperator(RdsBaseOperator):
     def _wait_until_db_available(self):
         self.log.info("Waiting for DB %s to reach 'available' state", self.db_type.value)
         if self.db_type == RdsDbType.INSTANCE:
-            self.hook.conn.get_waiter("db_instance_available").wait(DBInstanceIdentifier=self.db_identifier)
+            self.hook.wait_for_db_instance_state(self.db_identifier, target_state="available")
         else:
-            self.hook.conn.get_waiter("db_cluster_available").wait(DBClusterIdentifier=self.db_identifier)
+            self.hook.wait_for_db_cluster_state(self.db_identifier, target_state="available")
 
 
 class RdsStopDbOperator(RdsBaseOperator):
@@ -797,16 +718,10 @@ class RdsStopDbOperator(RdsBaseOperator):
 
     def _wait_until_db_stopped(self):
         self.log.info("Waiting for DB %s to reach 'stopped' state", self.db_type.value)
-        wait_statuses = ["stopping"]
-        ok_statuses = ["stopped"]
         if self.db_type == RdsDbType.INSTANCE:
-            self._await_status(
-                "db_instance", self.db_identifier, wait_statuses=wait_statuses, ok_statuses=ok_statuses
-            )
+            self.hook.wait_for_db_instance_state(self.db_identifier, target_state="stopped")
         else:
-            self._await_status(
-                "db_cluster", self.db_identifier, wait_statuses=wait_statuses, ok_statuses=ok_statuses
-            )
+            self.hook.wait_for_db_cluster_state(self.db_identifier, target_state="stopped")
 
 
 __all__ = [
