@@ -17,10 +17,12 @@
 # under the License.
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Sequence
+from typing import TYPE_CHECKING, Any, Sequence
 
 from airflow.compat.functools import cached_property
+from airflow.exceptions import AirflowException
 from airflow.providers.amazon.aws.hooks.ec2 import EC2Hook
+from airflow.providers.amazon.aws.triggers.ec2 import EC2StateSensorTrigger
 from airflow.sensors.base import BaseSensorOperator
 
 if TYPE_CHECKING:
@@ -39,6 +41,7 @@ class EC2InstanceStateSensor(BaseSensorOperator):
     :param target_state: target state of instance
     :param instance_id: id of the AWS EC2 instance
     :param region_name: (optional) aws region name associated with the client
+    :param deferrable: if True, the sensor will run in deferrable mode
     """
 
     template_fields: Sequence[str] = ("target_state", "instance_id", "region_name")
@@ -53,6 +56,7 @@ class EC2InstanceStateSensor(BaseSensorOperator):
         instance_id: str,
         aws_conn_id: str = "aws_default",
         region_name: str | None = None,
+        deferrable: bool = False,
         **kwargs,
     ):
         if target_state not in self.valid_states:
@@ -62,6 +66,22 @@ class EC2InstanceStateSensor(BaseSensorOperator):
         self.instance_id = instance_id
         self.aws_conn_id = aws_conn_id
         self.region_name = region_name
+        self.deferrable = deferrable
+
+    def execute(self, context: Context) -> Any:
+        if self.deferrable:
+            self.defer(
+                trigger=EC2StateSensorTrigger(
+                    instance_id=self.instance_id,
+                    target_state=self.target_state,
+                    aws_conn_id=self.aws_conn_id,
+                    region_name=self.region_name,
+                    poll_interval=int(self.poke_interval),
+                ),
+                method_name="execute_complete",
+            )
+        else:
+            super().execute(context=context)
 
     @cached_property
     def hook(self):
@@ -71,3 +91,8 @@ class EC2InstanceStateSensor(BaseSensorOperator):
         instance_state = self.hook.get_instance_state(instance_id=self.instance_id)
         self.log.info("instance state: %s", instance_state)
         return instance_state == self.target_state
+
+    def execute_complete(self, context, event=None):
+        if event["status"] != "success":
+            raise AirflowException(f"Error: {event}")
+        return
