@@ -15,8 +15,10 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
+
 import tempfile
-from typing import TYPE_CHECKING, Optional, Sequence, Union
+from typing import TYPE_CHECKING, Sequence
 
 from airflow.models import BaseOperator
 from airflow.providers.amazon.aws.hooks.glacier import GlacierHook
@@ -39,24 +41,13 @@ class GlacierToGCSOperator(BaseOperator):
         :ref:`howto/operator:GlacierToGCSOperator`
 
     :param aws_conn_id: The reference to the AWS connection details
-    :type aws_conn_id: str
     :param gcp_conn_id: The reference to the GCP connection details
-    :type gcp_conn_id: str
     :param vault_name: the Glacier vault on which job is executed
-    :type vault_name: string
     :param bucket_name: the Google Cloud Storage bucket where the data will be transferred
-    :type bucket_name: str
     :param object_name: the name of the object to check in the Google cloud
         storage bucket.
-    :type object_name: str
     :param gzip: option to compress local file or file data for upload
-    :type gzip: bool
-    :param chunk_size: size of chunk in bytes the that will downloaded from Glacier vault
-    :type chunk_size: int
-    :param delegate_to: The account to impersonate using domain-wide delegation of authority,
-        if any. For this to work, the service account making the request must have
-        domain-wide delegation enabled.
-    :type delegate_to: str
+    :param chunk_size: size of chunk in bytes the that will be downloaded from Glacier vault
     :param google_impersonation_chain: Optional Google service account to impersonate using
         short-term credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -65,7 +56,6 @@ class GlacierToGCSOperator(BaseOperator):
         If set as a sequence, the identities from the list must grant
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
-    :type google_impersonation_chain: Union[str, Sequence[str]]
     """
 
     template_fields: Sequence[str] = ("vault_name", "bucket_name", "object_name")
@@ -80,8 +70,7 @@ class GlacierToGCSOperator(BaseOperator):
         object_name: str,
         gzip: bool,
         chunk_size: int = 1024,
-        delegate_to: Optional[str] = None,
-        google_impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        google_impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -92,14 +81,12 @@ class GlacierToGCSOperator(BaseOperator):
         self.object_name = object_name
         self.gzip = gzip
         self.chunk_size = chunk_size
-        self.delegate_to = delegate_to
         self.impersonation_chain = google_impersonation_chain
 
-    def execute(self, context: 'Context') -> str:
+    def execute(self, context: Context) -> str:
         glacier_hook = GlacierHook(aws_conn_id=self.aws_conn_id)
         gcs_hook = GCSHook(
             gcp_conn_id=self.gcp_conn_id,
-            delegate_to=self.delegate_to,
             impersonation_chain=self.impersonation_chain,
         )
         job_id = glacier_hook.retrieve_inventory(vault_name=self.vault_name)
